package uniovi;

import java.io.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.apache.xerces.parsers.SAXParser;



public class NotaMediaSAX extends DefaultHandler {

    private float suma = 0;
    private float cuenta  = 0;  
    private StringBuffer contenido = new StringBuffer();
    private Locator locator;
    boolean enNota = false;

    public void setDocumentLocator(Locator loc) {
	locator = loc;
    }

    public void startElement(String namespaceURI, 
			   String localName,
			   String qName, 
			   Attributes atts) throws SAXException {
	if (localName.equals("nota")) {
	    enNota = true; 
	    contenido.setLength(0);
	}
    }

    public void characters(char [] cs, int inicio, int tam) throws SAXException {
	if (enNota) 
	    contenido.append(cs,inicio,tam);
    }

    public void endElement(String namespaceURI, 
		         String localName,
		         String qName)  throws SAXParseException {
	if (localName.equals("nota")) { 
	    try {
		float nota = new Float(contenido.toString()).floatValue();
		suma += nota;
		cuenta++;
	    } catch (java.lang.NumberFormatException err) {
		System.out.println("Error al convertir nota...");
		throw new SAXParseException("Error en " + locator.getSystemId() + 
				               ", lnea " + locator.getLineNumber() +
					  " . La nota debe ser un nmero",locator,err);
	    }
	enNota = false;
	}
    }

  public static void main(String[] args)  throws SAXException {

	if (args.length != 1) {
	    System.out.println("Forma de uso: NotaMedia ficheroAlumnos.xml");
	    System.exit(0);
	}
	String uri = args[0];
	System.out.println("Documento: " + uri);

    NotaMediaSAX n = new NotaMediaSAX();
    SAXParser p = new SAXParser();
    ErrorHandler eh = new GestorErrores();

    p.setContentHandler(n);
    p.setErrorHandler(eh);
    p.setFeature("http://xml.org/sax/features/validation",true);
    p.setFeature("http://xml.org/sax/features/namespaces",true);
    p.setFeature("http://apache.org/xml/features/validation/schema",true);

    try { 
     p.parse(uri); 
    } catch (Exception e) { 
	e.getMessage();
    }
    System.out.println("Nota Media (SAX) = " + (n.suma / n.cuenta));
  }
}

class GestorErrores implements ErrorHandler {
    public void warning(SAXParseException exc)  throws SAXException {
	System.out.println(" Aviso , Lnea: " + exc.getLineNumber() + ", URI = " + 
			   exc.getSystemId() + " Mensaje: " + exc.getMessage());
	throw new SAXException("Aviso...");
    }

    public void error(SAXParseException exc)  throws SAXException {
	System.out.println(" Error , Linea: " + exc.getLineNumber() + ", URI = " + 
			   exc.getSystemId() + " Mensaje: " + exc.getMessage());
	throw new SAXException("Error");
    }


    public void fatalError(SAXParseException exc)  throws SAXException {
	System.out.println(" Error fatal, Lnea: " + exc.getLineNumber() + ", URI = " + 
			   exc.getSystemId() + " Mensaje: " + exc.getMessage());
	throw new SAXException("Error fatal");
    }

}
